# Version, Author
__version__ = "0.0.1a4"
__author__ = "Luca Michael Schmidt"
__license__ = "MIT"

# importing of the classes
from BetterSockets.Client import Client
from BetterSockets.Server import HubServer, ProcessorServer
